package org.playground.base;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.persistence.CacheStoreMode;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import java.util.HashMap;
import java.util.Map;

@Component
public class TenantEntityManagerProvider {
	private final Logger LOGGER = LoggerFactory.getLogger(TenantEntityManagerProvider.class);

	private final Map<String, EntityManagerFactory> managerFactoryMap = new HashMap<>();
	private final ThreadLocal<Map<String, EntityManager>> threadTenantToEntityManagerMap = new ThreadLocal<>();

	@Value("${db.location}")
	private String location;

	private final String STATIC_TENANT_ID = "test";

	private synchronized EntityManagerFactory getEntityManagerFactory() {
		if(managerFactoryMap.containsKey(this.STATIC_TENANT_ID)) {
			return managerFactoryMap.get(this.STATIC_TENANT_ID);
		} else {
			EntityManagerFactory emf = Persistence.createEntityManagerFactory(location + "database.odb");
			managerFactoryMap.put(this.STATIC_TENANT_ID, emf);
			return emf;
		}
	}

	public synchronized EntityManager getEntityManager() {
		Map<String, EntityManager> tenantToEntityManagerMap = getTenantToEntityManagerMap();

		if(tenantToEntityManagerMap.containsKey(this.STATIC_TENANT_ID)) {
			return tenantToEntityManagerMap.get(this.STATIC_TENANT_ID);
		} else {
			EntityManager em = getEntityManagerFactory().createEntityManager();
			em.setProperty("javax.persistence.cache.storeMode", CacheStoreMode.REFRESH);
			tenantToEntityManagerMap.put(this.STATIC_TENANT_ID, em);
			return em;
		}
	}

	private synchronized Map<String, EntityManager> getTenantToEntityManagerMap() {
		Map<String, EntityManager> tenantToEntityManagerMap = threadTenantToEntityManagerMap.get();

		if(tenantToEntityManagerMap == null) {
			tenantToEntityManagerMap = new HashMap<>();
			threadTenantToEntityManagerMap.set(tenantToEntityManagerMap);
		}

		return tenantToEntityManagerMap;
	}

	/**
	 * Commits the active Transaction and closes it afterward
	 */
	public synchronized void commitTransaction() {
		if(getEntityManager().getTransaction().isActive()) {
			getEntityManager().flush();
			getEntityManager().getTransaction().commit();
			getEntityManager().close();
			getTenantToEntityManagerMap().remove(this.STATIC_TENANT_ID);
		}
	}

	/**
	 * Closes the active Transaction
	 */
	public synchronized void closeTransaction() {
		if(getEntityManager().getTransaction().isActive()) {
			getEntityManager().clear();
			getEntityManager().close();
			getTenantToEntityManagerMap().remove(this.STATIC_TENANT_ID);
		}
	}

	public synchronized void beginTransaction() {
		if(!getEntityManager().getTransaction().isActive()) {
			getEntityManager().getTransaction().begin();
		}
	}

	public synchronized void rollbackTransaction() {
		if(getEntityManager().getTransaction().isActive()) {
			getEntityManager().getTransaction().rollback();
			getEntityManager().close();
			getTenantToEntityManagerMap().remove(this.STATIC_TENANT_ID);
		}
	}

	public synchronized void closeAll() {
		for(EntityManagerFactory entityManagerFactory : managerFactoryMap.values()) {
			entityManagerFactory.close();
		}
		managerFactoryMap.clear();
	}
}
